#include "httpServerLib.h"

#include <openssl/ssl.h>
#include <openssl/err.h>

pthread_mutex_t *ssl_locks=(pthread_mutex_t *)0;
void ssl_locking_cb(int mode, int n, const char *file, int line)
{
(void)file; (void)line;
if(mode&CRYPTO_LOCK) { pthread_mutex_lock(ssl_locks+n); }
else { pthread_mutex_unlock(ssl_locks+n); }
}
unsigned long ssl_id_cb(void) { return (unsigned long)pthread_self(); }

//======================================================================
int recvLine(int sock, char *line, int lineSize)
{
	--lineSize;
	int len=0;
	while(len<lineSize)
	  {
		  int r=recv(sock,line+len,sizeof(char),0);
		  if(r<0) return -1;
		  if(!r||(line[len++]=='\n')) break;
	  }
	line[len]='\0';
	return len;
}

//======================================================================
void sendAll(int sock, const void *data, int dataSize)
{
	const char *ptr=(const char *)data;
	int remaining=dataSize;
	while(remaining)
	  {
		  int r=send(sock,ptr,remaining,0);
		  if(r<=0) break;
		  ptr+=r;
		  remaining-=r;
	  }
}

//======================================================================
// read bytes (bufferCapacity expected) or 0 (EOF)
int recvAll(int fd, void *buffer,int bufferCapacity)
{
  char *ptr=(char *)buffer;
  int remaining=bufferCapacity;
  while(remaining)
  {
    int r;
    RESTART_SYSCALL(r, (int)read(fd, ptr, (size_t)remaining));
    if(r<=0)
    {
      break;
    }
    ptr+=r;
    remaining-=r;
  }
  return bufferCapacity-remaining;
}

//======================================================================
// read bytes (bufferCapacity expected) or 0 (EOF)
int sslRecvAll(SSL *ssl, void *buffer,int bufferCapacity)
{
  char *ptr=(char *)buffer;
  int remaining=bufferCapacity;
  while(remaining)
  {
    int r;
    RESTART_SYSCALL(r, (int)SSL_read(ssl, ptr, (size_t)remaining));
    if(r<=0)
    {
      break;
    }
    ptr+=r;
    remaining-=r;
  }
  return bufferCapacity-remaining;
}

//======================================================================
int sslRecvLine(SSL *ssl, char *line,  int lineSize)
{
	--lineSize;
	int len=0;
	while(len<lineSize)
	  {
	  int r=SSL_read(ssl,line+len,sizeof(char));
	  if(r<0) return -1;
	  if(!r||(line[len++]=='\n')) break;
	  }
	line[len]='\0';
	return len;
}
//======================================================================
void sslSendAll(SSL *ssl, const void *data, int dataSize)
{
	const char *ptr=(const char *)data;
	int remaining=dataSize;
	while(remaining)
	  {
	  int r=SSL_write(ssl,ptr,remaining);
	  if(r<=0) break;
	  ptr+=r;
	  remaining-=r;
	  }
}
//======================================================================
Request * createRequest(int sock, bool useSsl)
{
	Request *req=(Request *)malloc(sizeof(Request));
		req->sock=sock;
		req->ssl=(SSL *)0;
		req->requestMethod[0]='\0';
		req->requestUri[0]='\0';
		req->contentLength[0]='\0';
		req->fileName[0]='\0';
		req->upgrade[0]='\0';	//###############################
		req->wsKey[0]='\0';		//###############################

		if(useSsl)
		  {
		  static SSL_CTX *sslCtx=(SSL_CTX *)0;
		  if(!sslCtx)
			{
				SSL_library_init();
				SSL_load_error_strings();
				OpenSSL_add_all_algorithms();
				sslCtx=SSL_CTX_new(SSLv23_method());
				SSL_CTX_use_certificate_file(sslCtx,"serverCert.pem",SSL_FILETYPE_PEM);
				SSL_CTX_use_PrivateKey_file(sslCtx,"serverKey.pem",SSL_FILETYPE_PEM);
				if(!SSL_CTX_check_private_key(sslCtx))
				  {
				  fprintf(stderr,"SSL_CTX_check_private_key: %s\n",
								 ERR_error_string(ERR_get_error(),NULL));
				  exit(1);
				  }
				ssl_locks=(pthread_mutex_t *)malloc(CRYPTO_num_locks()*
													sizeof(pthread_mutex_t));
				for(int i=CRYPTO_num_locks();i--;)
				  {
				  pthread_mutex_init(ssl_locks+i,(pthread_mutexattr_t *)0);
				  }
				CRYPTO_set_locking_callback(&ssl_locking_cb);
				CRYPTO_set_id_callback(&ssl_id_cb);
				}
			  req->ssl=SSL_new(sslCtx);
			  SSL_set_mode(req->ssl,SSL_MODE_AUTO_RETRY);
			  SSL_set_fd(req->ssl,sock);
			  if(SSL_accept(req->ssl)!=1)
				{
				fprintf(stderr,"SSL_accept: %s\n",
							   ERR_error_string(ERR_get_error(),NULL));
			}
		  }
	return req;
}

//======================================================================
void destroyRequest(Request *req)
{
 if(req->ssl)
  {
  SSL_shutdown(req->ssl);
  SSL_free(req->ssl);
  }
	close(req->sock);
	free(req);
}

//======================================================================
